/**
 *  @file fapiReturnCodes.H
 *
 *  @brief Defines the returns codes generated by HWPF code.
 */

/*
 * Change Log ******************************************************************
 * Flag     Defect/Feature  User        Date        Description
 * ------   --------------  ----------  ----------- ----------------------------
 *                          mjjones     04/13/2011  Created.
 *                          mjjones     06/07/2011  Add FAPI_RC_NOT_IMPLEMENTED
 *                          mjjones     06/30/2011  Add FAPI_RC_ATTR_UNIT_TEST_FAIL
 *
 */

#ifndef FAPIRETURNCODES_H_
#define FAPIRETURNCODES_H_

#include <ecmdDataBuffer.H>

namespace fapi
{

/**
 * @brief Enumeration of return codes
 */
enum
{
    FAPI_RC_SUCCESS = 0,

    // Flag bits indicating which code generated the error If no flag set then
    // it is generated by HWP
    FAPI_RC_FAPI_MASK = 0x04000000, // FAPI generated error
    FAPI_RC_PLAT_MASK = 0x02000000, // PLAT generated error
    FAPI_RC_ECMD_MASK = ECMD_ERR_ECMD, // ECMD generated error (0x01000000)

    // FAPI generated return codes
    FAPI_RC_NOT_IMPLEMENTED     = FAPI_RC_FAPI_MASK | 0x01,
    FAPI_RC_ATTR_UNIT_TEST_FAIL = FAPI_RC_FAPI_MASK | 0x02,

    // PLAT generated return codes
    FAPI_RC_PLAT_ERR_SEE_DATA    = FAPI_RC_PLAT_MASK | 0x01,
      // Error details in attached ReturnCodeData
    FAPI_RC_PLAT_NOT_IMPLEMENTED = FAPI_RC_PLAT_MASK | 0x02,
};

}

#endif // FAPIRETURNCODES_H_
